--[[
Name: LibLordFarlander-SpecialEvents-SpecialEvents_Skill-4.0
Author: LordFarlander
Original Author: Tekkub Stoutwrithe (tekkub@gmail.com)
Website: http://www.wowace.com/
Description: Special events for skills (learned, leveled, unlearned)
Dependencies: LibStub, CallbackHandler-1.0
]]--
--[[
Copyright (c) 2008-2012, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 256 $
-- $Date: 2012-09-30 17:26:25 +0000 (Sun, 30 Sep 2012) $

local vmajor, vminor = "LibLordFarlander-SpecialEvents-Skill-4.0", tonumber( ("$Revision: 256 $"):match( "(%d+)" ) ) + 90000

if( not LibStub( "LibLordFarlander-SpecialEvents-LearnSpell-4.0", true ) ) then
    error( vmajor .. " requires LibLordFarlander-SpecialEvents-LearnSpell-4.0" )
    return
end--if

local lib, oldMinor = LibStub:NewLibrary( vmajor, vminor )
if( not lib ) then
    return
end--if

local frame

if lib.frame then
    frame = lib.frame
    frame:UnregisterAllEvents()
    frame:SetScript( "OnEvent", nil )
else
    frame = CreateFrame( "Frame", vmajor .. "_Frame" )
    lib.frame = frame
end--if

frame:RegisterEvent( "CHAT_MSG_SKILL" )
frame:RegisterEvent( "CHAT_MSG_SYSTEM" )

frame:SetScript("OnEvent",
    function( self, event, ... )
        lib[event]( lib, ... )
    end )

if lib.callbacks then
    lib:UnregisterAll( lib )
else
    lib.callbacks = LibStub( "CallbackHandler-1.0" ):New( lib, nil, nil, "UnregisterAll" )
end--if

if( not lib.vars ) then
    lib.vars = {}
end--if

-- For safety's sake force the order
lib.vars.skillLevelNames = {
    [1] = "Apprentice",
    [2] = "Journeyman",
    [3] = "Expert",
    [4] = "Artisan",
    [5] = "Master",
    [6] = "Grand Master",
    [7] = "Illustrious",
    [8] = "Zen Master",
}

lib.vars.skillLevelValues = {
    ["Apprentice"] = 75,
    ["Journeyman"] = 150,
    ["Expert"] = 225,
    ["Artisan"] = 300,
    ["Master"] = 375,
    ["Grand Master"] = 450,
    ["Illustrious"] = 525,
    ["Zen Master"] = 600,
}

lib.vars.tradeIcons = {
    ["trade_alchemy"] = "Alchemy",
    ["trade_archaeology"] = "Archaeology",
    ["trade_blacksmithing"] = "Blacksmithing",
    ["inv_misc_food_15"] = "Cooking",
    ["trade_engraving"] = "Enchanting",
    ["trade_engineering"] = "Engineering",
    ["spell_holy_sealofsacrifice"] = "First Aid",
    ["trade_fishing"] = "Fishing",
    ["spell_nature_naturetouchgrow"] = "Herbalism",
    ["inv_inscription_tradeskill01"] = "Inscription",
    ["inv_misc_gem_01"] = "Jewelcrafting",
    ["inv_misc_armorkit_17"] = "Leatherworking",
    ["trade_mining"] = "Mining",
    ["spell_nature_swiftness"] = "Riding",
    ["inv_misc_pelt_wolf_01"] = "Skinning",
    ["trade_tailoring"] = "Tailoring",
}

lib.vars.skillIds = {
    -- ["Alchemy"] = {
        -- ["Elixir Master"] = 28677,
        -- ["Potion Master"] = 28675,
        -- ["Transmutation Master"] = 28672,
    -- },
    -- ["Blacksmithing"] = {
        -- ["Armorsmith"] = 9788,
        -- ["Weaponsmith"] = 9787,
        -- ["Master Axesmith"] = 17041,
        -- ["Master Hammersmith"] = 17040,
        -- ["Master Swordsmith"] = 17039,
    -- },
    -- ["Cooking"] = {
        -- ["Apprentice"] = 2550,
        -- ["Journeyman"] = 3102,
        -- ["Expert"] = 3413,
        -- ["Artisan"] = 18260,
        -- ["Master"] = 33359,
        -- ["Grand Master"] = 51296,
        -- ["Illustrious Grand Master"] = 88053,
    -- },
    -- ["Engineering"] = {
        -- ["Gnomish Engineer"] = 20219,
        -- ["Goblin Engineer"] = 20222,
    -- },
    -- ["Leatherworking"] = {
        -- ["Dragonscale Leatherworking"] = 10656,
        -- ["Elemental Leatherworking"] = 10658,
        -- ["Tribal Leatherworking"] = 10660,
    -- },
    ["Riding"] = {
        [33388] = "Apprentice",
        [33391] = "Journeyman",
        [34090] = "Expert",
        [34091] = "Artisan",
        [90265] = "Master",
    },
    -- ["Tailoring"] = {
        -- ["Mooncloth Tailoring"] = 26798,
        -- ["Shadoweave Tailoring"] = 26801,
        -- ["Spellfire Tailoring"] = 26797,
    -- },
}

lib.vars.specializations = {
    ["Alchemy"] = {
        [28677] = "Elixir Master",
        [28675] = "Potion Master",
        [28672] = "Transmutation Master",
    },
    ["Engineering"] = {
        [20219] = "Gnomish Engineer",
        [20222] = "Goblin Engineer",
    },
    ["Riding"] = {
        [130487] = "Cloud Serpent Riding",
        [54197] = "Cold Weather Flying",
        [90267] = "Flight Master's License",
        [115913] = "Wisdom of the Four Winds",
    },
}

--Create versions of the chat messages that we can use to scan
local SPECIALEVENTS_ERR_SKILL_GAINED_S = ERR_SKILL_GAINED_S:gsub( "%%s", "(.-)" )
local SPECIALEVENTS_ERR_SKILL_UP_SI = ERR_SKILL_UP_SI:gsub( "%%s", "(.-)" ):gsub( "%%1$s", "(.-)" ):gsub( "%%d", "(%%d+)" ):gsub( "%%2$d", "(%%d+)" )
local SPECIALEVENTS_ERR_SPELL_UNLEARNED_S = ERR_SPELL_UNLEARNED_S:gsub( "%%s", "(.-) %%(.-%%)" )

--------------------------------
--      Tracking methods      --
--------------------------------
function lib:CHAT_MSG_SYSTEM( message )
    --Figure out WHICH chat message was just fired and get some useful info about it
    local skillName = message:match( SPECIALEVENTS_ERR_SPELL_UNLEARNED_S )

    if( skillName ) then
        self.callbacks:Fire( "UnlearnedSkill", skillName )
        self.callbacks:Fire( "UnlearnedSkill_" .. skillName )
    end--if
end--lib:CHAT_MSG_SYSTEM( message )

function lib:CHAT_MSG_SKILL( message )
    --Figure out WHICH chat message was just fired and get some useful info about it
    local skillName, level

    skillName = message:match( SPECIALEVENTS_ERR_SKILL_GAINED_S )
    if( skillName ) then
        local skills = self:GetSkillList()

        self.callbacks:Fire( "LearnedSkill", skillName, skills[skillName] )
        self.callbacks:Fire( "LearnedSkill_" .. skillName, skills[skillName] )
    else
        skillName, level = message:match( SPECIALEVENTS_ERR_SKILL_UP_SI )

        if( skillName and level ) then
            level = tonumber( level )
            self.callbacks:Fire( "SkillLeveled", skillName, level )
            self.callbacks:Fire( "SkillLeveled_" .. skillName, level )
        end--if
    end--if
end--lib:CHAT_MSG_SKILL( message )

function lib:LearnedSpell( _, spell, _ )
    for level, ridingSpell in pairs( self.vars.skillIds["Riding"] ) do
        if( ridingSpell == spell ) then
            local skillLevel = self.vars.skillLevelValues[level]

            self.callbacks:Fire( "LearnedSkill", "Riding", skillLevel )
            self.callbacks:Fire( "LearnedSkill_Riding", skillLevel )
            break
        end--if
    end--for
end--lib:LearnedSpell( _, spell, _ )

function lib:GetSkillList()
    local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true )
    local skills = TR and TR:GetTable() or {}
    local namemap = TR and TR:GetTable() or {}
    local prof_1, prof_2, archaeology, fishing, cooking, firstaid = GetProfessions()
    local tradeIcons = self.vars.tradeIcons

    if( prof_1 ) then
        local name, icon, rank, maxrank, _, _, _ = GetProfessionInfo( prof_1 )
        local localName = tradeIcons[icon]

        skills[name] = rank
        if( localName ) then
            namemap[localName] = name
        end--if
    end--if
    if( prof_2 ) then
        local name, icon, rank, maxrank, _, _, _ = GetProfessionInfo( prof_2 )
        local localName = tradeIcons[icon]

        skills[name] = rank
        if( localName ) then
            namemap[localName] = name
        end--if
    end--if
    if( archaeology ) then
        local name, _, rank, maxrank, _, _, _ = GetProfessionInfo( archaeology )

        skills[name] = rank
        namemap["Archaeology"] = name
    end--if
    if( fishing ) then
        local name, _, rank, maxrank, _, _, _ = GetProfessionInfo( fishing )

        skills[name] = rank
        namemap["Fishing"] = name
    end--if
    if( cooking ) then
        local name, _, rank, maxrank, _, _, _ = GetProfessionInfo( cooking )

        skills[name] = rank
        namemap["Cooking"] = name
    end--if
    if( firstaid ) then
        local name, _, rank, maxrank, _, _, _ = GetProfessionInfo( firstaid )

        skills[name] = rank
        namemap["First Aid"] = name
    end--if
    if( self ) then
        local ridingSpecialEvents_Skill = self:GetRidingSkillLevel()

        if( ridingSpecialEvents_Skill ) then
            skills["Riding"] = ridingSpecialEvents_Skill
        end--if
    end--if
    return skills, namemap
end--lib:GetSkillList()

-----------------------------
--      Query methods      --
-----------------------------
function lib:GetSkills()
    return self:GetSkillList()
end--lib:GetSpecialEvents_Skills()

function lib:GetSkillLevel( skill )
    local skills = self:GetSkillList()

    return skills[skill]
end--lib:GetSkillLevel( skill )

function lib:GetRidingSkillLevel()
    local vars = self.vars
    local riding = vars.skillIds["Riding"]
    local skillLevelValues = vars.skillLevelValues
    local highest = nil
    local GetIdFromLink = function( link )
                              if( type( link ) == "string" ) then
                                  return tonumber( link:match( "^|c%x+|H(.+)|h%[.+%]" ):match( "(%d+)" ) )
                              end--if
                              return nil
                          end--GetIdFromLink( link )
    local _, _, _, numspells = GetSpellTabInfo( 1 )

    for i = 1, numspells do
        local name = GetSpellBookItemName( i, BOOKTYPE_SPELL )
        local id = GetIdFromLink( GetSpellLink( name ) )

        if( riding[id] ) then
            highest = highest or skillLevelValues[riding[id]]
        end--if
    end--for

    return highest
end--lib:GetRidingSkillLevel()

LibStub( "LibLordFarlander-SpecialEvents-LearnSpell-4.0" ).RegisterCallback( lib, "LearnedSpell" )
